import pytest

import test.mock.uv
from uv_ci_tools.app.get_executable import app
from uv_ci_tools.lib import ci, util

from ._util import ExecutableContext


def run_app(ci_type: ci.Type = ci.Type.GITLAB, project_name: str | None = 'project_name'):
    with (
        util.set_env('CI_PROJECT_PATH', 'project_path'),
        util.set_env('CI_COMMIT_REF_NAME', 'commit_ref_name'),
        util.set_env('CI_SERVER_HOST', 'server_host'),
    ):
        app.get_executable(ci_type=ci_type, project_name=project_name)


def test_simple(capsys: pytest.CaptureFixture[str]):
    with ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)

        with test.mock.uv.set_uv_tool_list_show_paths_output(
            'project_name v0.0.0 (/)\n- tool (executable_path)'
        ):
            run_app()
    assert capsys.readouterr().out == 'executable_path'


def test_no_tool_installed():
    with ExecutableContext.make() as exe_ctx:
        exe_ctx.add_executable(test.mock.uv)

        with (
            test.mock.uv.set_uv_tool_list_show_paths_output('project_name v0.0.0 (/)'),
            pytest.raises(RuntimeError, match='No executable available'),
        ):
            run_app()



import pathlib
import typing

import tomlkit.items

from uv_ci_tools.lib import ver


def get_path():
    return pathlib.Path('pyproject.toml')


def update_pyproject_version(version_action: typing.Callable[[ver.Version], ver.Version]):
    pyproject_path = get_path()
    pyproject_document = tomlkit.parse(pyproject_path.read_text())
    project_item = pyproject_document['project']
    assert isinstance(project_item, tomlkit.items.Table)
    version_item = project_item['version']
    assert isinstance(version_item, tomlkit.items.String)
    old_version = ver.Version.load(version_item)
    new_version = version_action(old_version)
    project_item['version'] = new_version.dump()
    pyproject_path.write_text(pyproject_document.as_string())
    return new_version



from uv_ci_tools.lib import cli

from . import bump_version, get_executable, pre_compile

APP = cli.main_app()

APP.command(bump_version.APP)
APP.command(pre_compile.APP)
APP.command(get_executable.APP)



import sys

from uv_ci_tools.lib import ci, cli, uv

APP = cli.sub_app(__name__)


@APP.default
def get_executable(*, ci_type: ci.Type = ci.Type.GITLAB, project_name: str | None = None):
    ci_ctx = ci_type.fill_context(ci.PartialContext(project_name=project_name))
    installed_package = uv.get_installed_package(ci_ctx.project_name)

    first_tool = next(iter(installed_package.tools), None)
    if first_tool is None:
        msg = f'No executable available for {project_name}'
        raise RuntimeError(msg)

    sys.stdout.write(str(first_tool.path))
